"""Concept module for mapping a concept to rdf.

This module contains methods for mapping a concept object to rdf
according to the
`skos-ap-no standard <https://doc.difi.no/data/begrep-skos-ap-no/#_begrep>`__

Example:
    >>> from concepttordf import Concept, Definition
    >>>
    >>> concept = Concept()
    >>> concept.identifier = "http://example.com/concepts/1"
    >>> concept.term = {"name": {"en": "concept"}}
    >>>
    >>> definition = Definition()
    >>> definition.text = {"text": {"en": "abstract or generic idea"}}
    >>> concept.definition = definition
    >>>
    >>> bool(concept.to_rdf())
    True
"""
from __future__ import annotations

from datetime import date
from typing import List

from rdflib import BNode, Graph, Literal, Namespace, RDF, RDFS, URIRef

from .alternativformulering import AlternativFormulering
from .associativerelation import AssociativeRelation
from .betydningsbeskrivelse import Betydningsbeskrivelse, RelationToSource
from .contact import Contact
from .definition import Definition
from .genericrelation import GenericRelation
from .partitiverelation import PartitiveRelation


DCT = Namespace("http://purl.org/dc/terms/")
SKOSXL = Namespace("http://www.w3.org/2008/05/skos-xl#")
VCARD = Namespace("http://www.w3.org/2006/vcard/ns#")
SKOS = Namespace("http://www.w3.org/2004/02/skos/core#")
SKOSNO = Namespace("https://data.norge.no/vocabulary/skosno#")
DCAT = Namespace("http://www.w3.org/ns/dcat#")
XSD = Namespace("http://www.w3.org/2001/XMLSchema#")
SCHEMA = Namespace("http://schema.org/")
XKOS = Namespace("http://rdf-vocabulary.ddialliance.org/xkos#")


class Concept:
    """A class representing a concept.

    Attributes:
        identifier: the uri identifying the concept
        term: a dictionary describing the prefLabel
        alternativeterm: a dictionary describing altLabel
        hiddenterm: a dictionary describing the hiddenLabel
    """

    # Types
    _identifier: str
    _term: dict
    _alternativeterm: dict
    _hiddenterm: dict
    _datastrukturterm: dict
    _subject: dict
    _definition: Definition
    _alternativformulering: AlternativFormulering
    _contactpoint: Contact
    _modified: date
    _bruksomrade: dict
    _validinperiod: dict
    _publisher: str
    _seeAlso: List[Concept]
    _replaces: List[Concept]
    _replacedBy: List[Concept]
    _related: AssociativeRelation
    _generalizes: GenericRelation
    _specializes: GenericRelation
    _has_part: PartitiveRelation
    _is_part_of: PartitiveRelation
    _dct_identifier: str

    def __init__(self) -> None:
        """Inits an object with default values."""
        self._g = Graph()

        self.seeAlso = []
        self.replaces = []
        self.replacedBy = []

    @property
    def identifier(self: Concept) -> str:
        """Identifier attribute.

        Returns:
            a string holding a valid uri
        """
        return self._identifier

    @identifier.setter
    def identifier(self, uri: str) -> None:
        self._identifier = uri

    @property
    def term(self: Concept) -> dict:
        """Term attribute."""
        return self._term

    @term.setter
    def term(self: Concept, term: dict) -> None:
        self._term = term

    @property
    def alternativeterm(self: Concept) -> dict:
        """Alternativeterm attribute."""
        return self._alternativeterm

    @alternativeterm.setter
    def alternativeterm(self: Concept, alternativeterm: dict) -> None:
        self._alternativeterm = alternativeterm

    @property
    def hiddenterm(self: Concept) -> dict:
        """Hiddenterm attribute."""
        return self._hiddenterm

    @hiddenterm.setter
    def hiddenterm(self: Concept, hiddenterm: dict) -> None:
        self._hiddenterm = hiddenterm

    @property
    def datastrukturterm(self: Concept) -> dict:
        """Datastrukturterm attribute."""
        return self._datastrukturterm

    @datastrukturterm.setter
    def datastrukturterm(self: Concept, datastrukturterm: dict) -> None:
        self._datastrukturterm = datastrukturterm

    @property
    def subject(self: Concept) -> dict:
        """Subject attribute."""
        return self._subject

    @subject.setter
    def subject(self: Concept, subject: dict) -> None:
        self._subject = subject

    @property
    def definition(self: Concept) -> Definition:
        """Definition attribute."""
        return self._definition

    @definition.setter
    def definition(self: Concept, definition: Definition) -> None:
        self._definition = definition

    @property
    def alternativformulering(self: Concept) -> AlternativFormulering:
        """Alternativformulering attribute."""
        return self._alternativformulering

    @alternativformulering.setter
    def alternativformulering(
        self: Concept, alternativformulering: AlternativFormulering
    ) -> None:
        self._alternativformulering = alternativformulering

    @property
    def contactpoint(self: Concept) -> Contact:
        """Contacpoint attribute."""
        return self._contactpoint

    @contactpoint.setter
    def contactpoint(self: Concept, contact: Contact) -> None:
        self._contactpoint = contact

    @property
    def modified(self: Concept) -> date:
        """Modified attribute."""
        return self._modified

    @modified.setter
    def modified(self: Concept, modified: date) -> None:
        self._modified = modified

    @property
    def bruksomrade(self: Concept) -> dict:
        """Bruksomrade attribute."""
        return self._bruksomrade

    @bruksomrade.setter
    def bruksomrade(self: Concept, bruksomrade: dict) -> None:
        self._bruksomrade = bruksomrade

    @property
    def validinperiod(self: Concept) -> dict:
        """Validinperiod attribute."""
        return self._validinperiod

    @validinperiod.setter
    def validinperiod(self: Concept, validinperiod: dict) -> None:
        self._validinperiod = validinperiod

    @property
    def publisher(self: Concept) -> str:
        """Publisher attribute."""
        return self._publisher

    @publisher.setter
    def publisher(self: Concept, publisher: str) -> None:
        self._publisher = publisher

    @property
    def seeAlso(self: Concept) -> List[Concept]:
        """Seealso attribute."""
        return self._seeAlso

    @seeAlso.setter
    def seeAlso(self: Concept, concepts: List[Concept]) -> None:
        self._seeAlso = concepts

    @property
    def replaces(self: Concept) -> List[Concept]:
        """Replaces attribute."""
        return self._replaces

    @replaces.setter
    def replaces(self: Concept, concepts: List[Concept]) -> None:
        self._replaces = concepts

    @property
    def replacedBy(self: Concept) -> List[Concept]:
        """Replacedby attribute."""
        return self._replacedBy

    @replacedBy.setter
    def replacedBy(self: Concept, concepts: List[Concept]) -> None:
        self._replacedBy = concepts

    @property
    def related(self: Concept) -> AssociativeRelation:
        """Related attribute."""
        return self._related

    @related.setter
    def related(self: Concept, ar: AssociativeRelation) -> None:
        self._related = ar

    @property
    def generalizes(self: Concept) -> GenericRelation:
        """Generalizes attribute."""
        return self._generalizes

    @generalizes.setter
    def generalizes(self: Concept, gr: GenericRelation) -> None:
        self._generalizes = gr

    @property
    def specializes(self: Concept) -> GenericRelation:
        """Specializes attribute."""
        return self._specializes

    @specializes.setter
    def specializes(self: Concept, gr: GenericRelation) -> None:
        self._specializes = gr

    @property
    def has_part(self: Concept) -> PartitiveRelation:
        """Has part attribute."""
        return self._has_part

    @has_part.setter
    def has_part(self: Concept, gr: PartitiveRelation) -> None:
        self._has_part = gr

    @property
    def is_part_of(self: Concept) -> PartitiveRelation:
        """Is part of attribute."""
        return self._is_part_of

    @is_part_of.setter
    def is_part_of(self: Concept, gr: PartitiveRelation) -> None:
        self._is_part_of = gr

    @property
    def dct_identifier(self) -> str:
        """Get for dct_identifier."""
        return self._dct_identifier

    @dct_identifier.setter
    def dct_identifier(self, dct_identifier: str) -> None:
        """Set for dct_identifier."""
        self._dct_identifier = dct_identifier

    # ----------------------------------------------

    def _to_graph(self) -> Graph:
        """Transforms the concept to an rdf graph.

        Returns:
            an RDF graph representing the concept.
        """
        self._add_concept_to_graph()

        return self._g

    def to_rdf(self: Concept, format: str = "text/turtle") -> str:
        """Maps the concept to rdf.

        Args:
            format: a valid serialization format.

        Returns:
            a serialization of the RDF graph
        """
        return self._to_graph().serialize(format=format)

    # ----------------------------------------------

    def _add_concept_to_graph(self: Concept) -> None:
        """Adds the concept to the Graph _g."""
        self._g.bind("dct", DCT)
        self._g.bind("skos", SKOS)
        self._g.bind("skosxl", SKOSXL)
        self._g.bind("vcard", VCARD)
        self._g.bind("skosno", SKOSNO)
        self._g.bind("dcat", DCAT)
        self._g.bind("xsd", XSD)
        self._g.bind("schema", SCHEMA)
        self._g.bind("xkos", XKOS)

        self._g.add((URIRef(self.identifier), RDF.type, SKOS.Concept))

        # dct:identifier
        self._dct_identifier_to_graph()

        # prefLabel
        self._term_to_graph()

        # altLabel
        self._add_alternativeterm_to_graph()

        # hiddenLabel
        self._add_hiddenterm_to_graph()

        # datastrukturterm
        self._add_datastrukturterm_to_graph()

        # definition
        if getattr(self, "definition", None):
            self._add_betydningsbeskrivelse_to_concept(self.definition)

        # alternativformulering
        if getattr(self, "alternativformulering", None):
            self._add_betydningsbeskrivelse_to_concept(self.alternativformulering)

        # publisher
        self._add_publisher_to_graph()

        # contactPoint
        self._add_contactpoint_to_graph()

        # subject
        self._add_subject_to_graph()

        # modified
        self._add_modified_to_graph()

        # bruksomrade
        self._add_bruksomrade_to_graph()

        # PeriodOfTime
        self._add_validinperiod_to_graph()

        # seeAlso
        self._add_seeAlso_to_graph()

        # replaces
        self._add_replaces_to_graph()

        # replacedBy
        self._add_replacedBy_to_graph()

        # related
        self._add_related_to_graph()

        # generalizes
        self._add_generalizes_to_graph()

        # specializes
        self._add_specializes_to_graph()

        # has_part
        self._add_has_part_to_graph()

        # is_part_of
        self._add_is_part_of_to_graph()

    # ------------
    # Helper methods:

    def _dct_identifier_to_graph(self: Concept) -> None:
        if getattr(self, "dct_identifier", None):
            self._g.add(
                (
                    URIRef(self.identifier),
                    DCT.identifier,
                    Literal(self.dct_identifier),
                )
            )

    def _term_to_graph(self: Concept) -> None:
        if getattr(self, "term", None):
            label = BNode()
            self._g.add((label, RDF.type, SKOSXL.Label))
            if "name" in self.term:
                _name = self.term["name"]
                for key in _name:
                    self._g.add(
                        (label, SKOSXL.literalForm, Literal(_name[key], lang=key))
                    )
            if "modified" in self.term:
                self._g.add(
                    (
                        label,
                        DCT.modified,
                        Literal(self.term["modified"], datatype=XSD.date),
                    )
                )
            self._g.add((URIRef(self.identifier), SKOSXL.prefLabel, label))

    def _add_alternativeterm_to_graph(self: Concept) -> None:
        if getattr(self, "alternativeterm", None):
            altLabel = BNode()
            self._g.add((altLabel, RDF.type, SKOSXL.Label))
            if "name" in self.alternativeterm:
                _name = self.alternativeterm["name"]
                for key in _name:
                    for _l in _name[key]:
                        self._g.add(
                            (altLabel, SKOSXL.literalForm, Literal(_l, lang=key))
                        )
            if "modified" in self.alternativeterm:
                self._g.add(
                    (
                        altLabel,
                        DCT.modified,
                        Literal(self.alternativeterm["modified"], datatype=XSD.date),
                    )
                )
            self._g.add((URIRef(self.identifier), SKOSXL.altLabel, altLabel))

    def _add_datastrukturterm_to_graph(self: Concept) -> None:
        if getattr(self, "datastrukturterm", None):
            datastrukturterm = BNode()
            self._g.add((datastrukturterm, RDF.type, SKOSXL.Label))
            if "name" in self.datastrukturterm:
                _name = self.datastrukturterm["name"]
                for key in _name:
                    for _l in _name[key]:
                        self._g.add(
                            (
                                datastrukturterm,
                                SKOSXL.literalForm,
                                Literal(_l, lang=key),
                            )
                        )
            if "modified" in self.datastrukturterm:
                self._g.add(
                    (
                        datastrukturterm,
                        DCT.modified,
                        Literal(self.datastrukturterm["modified"], datatype=XSD.date),
                    )
                )
            self._g.add(
                (URIRef(self.identifier), SKOSNO.datastrukturterm, datastrukturterm)
            )

    def _add_hiddenterm_to_graph(self: Concept) -> None:
        if getattr(self, "hiddenterm", None):
            hiddenLabel = BNode()
            self._g.add((hiddenLabel, RDF.type, SKOSXL.Label))
            if "name" in self.hiddenterm:
                _name = self.hiddenterm["name"]
                for key in _name:
                    for _l in _name[key]:
                        self._g.add(
                            (hiddenLabel, SKOSXL.literalForm, Literal(_l, lang=key))
                        )
            if "modified" in self.hiddenterm:
                self._g.add(
                    (
                        hiddenLabel,
                        DCT.modified,
                        Literal(self.hiddenterm["modified"], datatype=XSD.date),
                    )
                )
            self._g.add((URIRef(self.identifier), SKOSXL.hiddenLabel, hiddenLabel))

    def _add_contactpoint_to_graph(self: Concept) -> None:
        if getattr(self, "contactpoint", None):
            contact = self.contactpoint
            contactPoint = BNode()
            for _s, p, o in contact._to_graph().triples((None, None, None)):
                self._g.add((contactPoint, p, o))
            self._g.add((URIRef(self.identifier), DCAT.contactPoint, contactPoint))

    def _add_subject_to_graph(self: Concept) -> None:
        if getattr(self, "subject", None):
            for key in self.subject:
                self._g.add(
                    (
                        URIRef(self.identifier),
                        DCT.subject,
                        Literal(self.subject[key], lang=key),
                    )
                )

    def _add_bruksomrade_to_graph(self: Concept) -> None:
        if getattr(self, "bruksomrade", None):
            for key in self.bruksomrade:
                for b in self.bruksomrade[key]:
                    self._g.add(
                        (
                            URIRef(self.identifier),
                            SKOSNO.bruksområde,
                            Literal(b, lang=key),
                        )
                    )

    def _add_validinperiod_to_graph(self: Concept) -> None:
        if getattr(self, "validinperiod", None):
            periodOfTime = BNode()
            self._g.add((periodOfTime, RDF.type, DCT.PeriodOfTime))
            if "startdate" in self.validinperiod:
                self._g.add(
                    (
                        periodOfTime,
                        SCHEMA.startDate,
                        Literal(self.validinperiod["startdate"], datatype=XSD.date),
                    )
                )
            if "enddate" in self.validinperiod:
                self._g.add(
                    (
                        periodOfTime,
                        SCHEMA.endDate,
                        Literal(self.validinperiod["enddate"], datatype=XSD.date),
                    )
                )
            self._g.add((URIRef(self.identifier), DCT.temporal, periodOfTime))

    def _add_publisher_to_graph(self: Concept) -> None:
        if getattr(self, "publisher", None):
            self._g.add(
                (URIRef(self.identifier), DCT.publisher, URIRef(self.publisher))
            )

    def _add_modified_to_graph(self: Concept) -> None:
        if getattr(self, "modified", None):
            self._g.add(
                (
                    URIRef(self.identifier),
                    DCT.modified,
                    Literal(self.modified, datatype=XSD.date),
                )
            )

    def _add_seeAlso_to_graph(self: Concept) -> None:
        if getattr(self, "seeAlso", None):
            for c in self.seeAlso:
                self._g.add(
                    (URIRef(self.identifier), RDFS.seeAlso, URIRef(c.identifier))
                )

    def _add_replaces_to_graph(self: Concept) -> None:
        if getattr(self, "replaces", None):
            for c in self.replaces:
                self._g.add(
                    (URIRef(self.identifier), DCT.replaces, URIRef(c.identifier))
                )

    def _add_replacedBy_to_graph(self: Concept) -> None:
        if getattr(self, "replacedBy", None):
            for c in self.replacedBy:
                self._g.add(
                    (URIRef(self.identifier), DCT.replacedBy, URIRef(c.identifier))
                )

    def _add_related_to_graph(self: Concept) -> None:
        if getattr(self, "related", None):
            _related = self.related
            ar = BNode()
            for _s, p, o in _related._to_graph().triples((None, None, None)):
                self._g.add((ar, p, o))
            self._g.add((URIRef(self.identifier), SKOSNO.assosiativRelasjon, ar))

    def _add_generalizes_to_graph(self: Concept) -> None:
        if getattr(self, "generalizes", None):
            _generalizes = self.generalizes
            ar = BNode()
            for _s, p, o in _generalizes._to_graph().triples((None, None, None)):
                self._g.add((ar, p, o))
            self._g.add((URIRef(self.identifier), SKOSNO.generiskRelasjon, ar))

    def _add_specializes_to_graph(self: Concept) -> None:
        if getattr(self, "specializes", None):
            _specializes = self.specializes
            ar = BNode()
            for _s, p, o in _specializes._to_graph().triples((None, None, None)):
                self._g.add((ar, p, o))
            self._g.add((URIRef(self.identifier), SKOSNO.generiskRelasjon, ar))

    def _add_has_part_to_graph(self: Concept) -> None:
        if getattr(self, "has_part", None):
            _has_part = self.has_part
            ar = BNode()
            for _s, p, o in _has_part._to_graph().triples((None, None, None)):
                self._g.add((ar, p, o))
            self._g.add((URIRef(self.identifier), SKOSNO.partitivRelasjon, ar))

    def _add_is_part_of_to_graph(self: Concept) -> None:
        if getattr(self, "is_part_of", None):
            _is_part_of = self.is_part_of
            ar = BNode()
            for _s, p, o in _is_part_of._to_graph().triples((None, None, None)):
                self._g.add((ar, p, o))
            self._g.add((URIRef(self.identifier), SKOSNO.partitivRelasjon, ar))

    # -
    def _add_betydningsbeskrivelse_to_concept(
        self: Concept, betydningsbeskrivelse: Betydningsbeskrivelse
    ) -> None:

        _betydningsbeskrivelse = BNode()

        # Check type of sub class, and add corresponding object:
        if isinstance(betydningsbeskrivelse, Definition):
            self._g.add((_betydningsbeskrivelse, RDF.type, SKOSNO.Definisjon))
        else:
            self._g.add(
                (_betydningsbeskrivelse, RDF.type, SKOSNO.AlternativFormulering)
            )

        # text
        self._add_text_to_bs_graph(betydningsbeskrivelse, _betydningsbeskrivelse)

        # remark
        self._add_remark_to_bs_graph(betydningsbeskrivelse, _betydningsbeskrivelse)

        # scope
        self._add_scope_to_bs_graph(betydningsbeskrivelse, _betydningsbeskrivelse)

        # relationtosource
        self._add_relationtosource_bs_to_graph(
            betydningsbeskrivelse, _betydningsbeskrivelse
        )

        # source
        self._add_source_to_bs_graph(betydningsbeskrivelse, _betydningsbeskrivelse)

        # modified
        self._add_modified_to_bs_graph(betydningsbeskrivelse, _betydningsbeskrivelse)

        # example
        self._add_example_to_bs_graph(betydningsbeskrivelse, _betydningsbeskrivelse)

        # Check type, and set correct property
        if isinstance(betydningsbeskrivelse, Definition):
            self._g.add(
                (URIRef(self.identifier), SKOSNO.definisjon, _betydningsbeskrivelse)
            )
        else:
            self._g.add(
                (
                    URIRef(self.identifier),
                    SKOSNO.alternativFormulering,
                    _betydningsbeskrivelse,
                )
            )
        # ---

    def _add_text_to_bs_graph(
        self: Concept, betydningsbeskrivelse: Betydningsbeskrivelse, bsnode: BNode
    ) -> None:
        if getattr(betydningsbeskrivelse, "text", None):
            _text = betydningsbeskrivelse.text
            for key in _text:
                self._g.add(
                    (
                        bsnode,
                        RDFS.label,
                        Literal(_text[key], lang=key),
                    )
                )

    def _add_remark_to_bs_graph(
        self: Concept, betydningsbeskrivelse: Betydningsbeskrivelse, bsnode: BNode
    ) -> None:
        if getattr(betydningsbeskrivelse, "remark", None):
            _remark = betydningsbeskrivelse.remark
            for key in _remark:
                self._g.add(
                    (
                        bsnode,
                        SKOS.scopeNote,
                        Literal(betydningsbeskrivelse.remark[key], lang=key),
                    )
                )

    def _add_scope_to_bs_graph(
        self: Concept, betydningsbeskrivelse: Betydningsbeskrivelse, bsnode: BNode
    ) -> None:
        if getattr(betydningsbeskrivelse, "scope", None):
            _scope = BNode()
            if "url" in betydningsbeskrivelse.scope:
                self._g.add(
                    (
                        _scope,
                        RDFS.seeAlso,
                        URIRef(betydningsbeskrivelse.scope["url"]),
                    )
                )
            if "text" in betydningsbeskrivelse.scope:
                _text = betydningsbeskrivelse.scope["text"]
                for key in _text:
                    self._g.add((_scope, RDFS.label, Literal(_text[key], lang=key)))
            self._g.add((bsnode, SKOSNO.omfang, _scope))

    def _add_relationtosource_bs_to_graph(
        self: Concept, betydningsbeskrivelse: Betydningsbeskrivelse, bsnode: BNode
    ) -> None:
        if getattr(betydningsbeskrivelse, "relationtosource", None):
            # -
            # sitatFraKilde = "quoteFromSource"
            # basertPaKilde = "basedOnSource"
            # egendefinert = "noSource"
            # -
            if (
                RelationToSource(betydningsbeskrivelse.relationtosource)
                is RelationToSource.sitatFraKilde
            ):
                self._g.add(
                    (
                        bsnode,
                        SKOSNO.forholdTilKilde,
                        SKOSNO.sitatFraKilde,
                    )
                )
            elif (
                RelationToSource(betydningsbeskrivelse.relationtosource)
                is RelationToSource.basertPaKilde
            ):
                self._g.add(
                    (
                        bsnode,
                        SKOSNO.forholdTilKilde,
                        SKOSNO.basertPåKilde,
                    )
                )
            else:
                self._g.add(
                    (
                        bsnode,
                        SKOSNO.forholdTilKilde,
                        SKOSNO.egendefinert,
                    )
                )

    def _add_source_to_bs_graph(
        self: Concept, betydningsbeskrivelse: Betydningsbeskrivelse, bsnode: BNode
    ) -> None:
        if getattr(betydningsbeskrivelse, "source", None):
            _source = BNode()
            if "url" in betydningsbeskrivelse.source:
                self._g.add(
                    (
                        _source,
                        RDFS.seeAlso,
                        URIRef(betydningsbeskrivelse.source["url"]),
                    )
                )
            if "text" in betydningsbeskrivelse.source:
                _text = betydningsbeskrivelse.source["text"]
                for key in _text:
                    self._g.add((_source, RDFS.label, Literal(_text[key], lang=key)))
            self._g.add((bsnode, DCT.source, _source))

    def _add_modified_to_bs_graph(
        self: Concept, betydningsbeskrivelse: Betydningsbeskrivelse, bsnode: BNode
    ) -> None:
        if getattr(betydningsbeskrivelse, "modified", None):
            self._g.add(
                (
                    bsnode,
                    DCT.modified,
                    Literal(betydningsbeskrivelse.modified, datatype=XSD.date),
                )
            )

    def _add_example_to_bs_graph(
        self: Concept, betydningsbeskrivelse: Betydningsbeskrivelse, bsnode: BNode
    ) -> None:
        if getattr(betydningsbeskrivelse, "example", None):
            _example = betydningsbeskrivelse.example
            for key in _example:
                self._g.add(
                    (
                        bsnode,
                        SKOS.example,
                        Literal(_example[key], lang=key),
                    )
                )



"""Contact module for mapping a dataService to rdf.

This module contains methods for mapping a contact object to rdf
according to the
`vcard ontology <https://www.w3.org/2006/vcard/ns-2006.html>`__

Example:
    >>> from concepttordf import Contact
    >>>
    >>> contact = Contact()
    >>> contact.identifier = "http://example.com/contacts/1"
    >>> contact.name = {'nb': 'Digitaliseringsdirektoratet'}
    >>> bool(contact.to_rdf())
    True
"""
from __future__ import annotations

from rdflib import BNode, Graph, Literal, Namespace, RDF, URIRef

VCARD = Namespace("http://www.w3.org/2006/vcard/ns#")


class Contact:
    """A class representing a vcard:Contact.

    Attributes:
        identifier: unique uri for contact
        name: name of contact in various languages
        email: email as a mailto link
        telephone: telephone number
        url: url to website
    """

    _identifier: str
    _name: dict
    _email: str
    _telephone: str
    _url: str

    def __init__(self) -> None:
        """Inits an object with default values."""
        self._g = Graph()

    @property
    def identifier(self) -> str:
        """Identifier attribute.

        Returns:
            a string holding a valid uri
        """
        return self._identifier

    @identifier.setter
    def identifier(self, uri: str) -> None:
        self._identifier = uri

    @property
    def name(self) -> dict:
        """Get/set for name."""
        return self._name

    @name.setter
    def name(self, name: dict) -> None:
        self._name = name

    @property
    def email(self) -> str:
        """Get/set for email."""
        return self._email

    @email.setter
    def email(self, email: str) -> None:
        self._email = email

    @property
    def telephone(self) -> str:
        """Get/set for telephone."""
        return self._telephone

    @telephone.setter
    def telephone(self, telephone: str) -> None:
        self._telephone = telephone

    @property
    def url(self) -> str:
        """Get/set for url."""
        return self._url

    @url.setter
    def url(self, url: str) -> None:
        self._url = url

    def _to_graph(self) -> Graph:

        self._add_contact_to_graph()

        return self._g

    def to_rdf(self, format: str = "text/turtle") -> str:
        """Maps the contact to rdf.

        Available formats:
         - turtle (default)
         - xml
         - json-ld

        Args:
            format: a valid format.

        Returns:
            a rdf serialization as a string according to format.
        """
        return self._to_graph().serialize(format=format)

    # -----

    def _add_contact_to_graph(self: Contact) -> None:
        """Adds the concept to the Graph _g."""
        self._g.bind("vcard", VCARD)

        if getattr(self, "identifier", None):
            _self = URIRef(self.identifier)
        else:
            _self = BNode()  # type: ignore

        self._g.add((_self, RDF.type, VCARD.Organization))

        # name
        if getattr(self, "name", None):
            for key in self.name:
                self._g.add(
                    (
                        _self,
                        VCARD.hasOrganizationName,
                        Literal(self.name[key], lang=key),
                    )
                )

        # email
        if getattr(self, "email", None):
            self._g.add((_self, VCARD.hasEmail, URIRef("mailto:" + self.email)))

        # telephone
        if getattr(self, "telephone", None):
            self._g.add((_self, VCARD.hasTelephone, URIRef("tel:" + self.telephone)))

        # url
        if getattr(self, "url", None):
            self._g.add((_self, VCARD.hasURL, URIRef(self.url)))



"""Concept module for mapping abstract class ConceptRelation to rdf."""
from abc import ABC
from datetime import date

from rdflib import BNode, Graph, Literal, Namespace

SKOSNO = Namespace("https://data.norge.no/vocabulary/skosno#")
DCT = Namespace("http://purl.org/dc/terms/")
XSD = Namespace("http://www.w3.org/2001/XMLSchema#")


class ConceptRelation(ABC):
    """An abstract class representing a concept relation.

    Attributes:
        modified: date when relation was last modified
    """

    _modified: date

    def __init__(self) -> None:
        """Inits an object with default values."""
        self._g = Graph()
        self._relation = BNode()
        self._g.bind("skosno", SKOSNO)
        self._g.bind("dct", DCT)
        self._g.bind("xsd", XSD)

    @property
    def modified(self) -> date:
        """Modified attribute."""
        return self._modified

    @modified.setter
    def modified(self, modified: date) -> None:
        """Modified attribute.

        Args:
            modified: e.g. "yyyy-mm-dd"
        """
        self._modified = modified

    # ---
    def _add_relation_to_graph(self) -> None:

        # modified
        if getattr(self, "modified", None):
            self._g.add(
                (
                    self._relation,
                    DCT.modified,
                    Literal(self.modified, datatype=XSD.date),
                )
            )



"""Concept collection to rdf.

This library maps a concept collection to skos according to
[SKOS-AP-NO](https://doc.difi.no/data/begrep-skos-ap-no/),
a Norwegian application profile for [SKOS](https://www.w3.org/TR/skos-primer/).

The library wraps [rdflib](https://rdflib.readthedocs.io/en/stable/).
The following serialization formats are available:

 - `text/turtle`
 - `application/rdf+xml`
 - `application/ld+json`
 - `application/n-triples`
 - `text/n3`

Example:
    >>> from concepttordf import Collection, Concept, Definition
    >>>
    >>> # Create collection object
    >>> collection = Collection()
    >>> collection.identifier = "http://example.com/collections/1"
    >>> collection.name = {"en": "A concept collection"}
    >>> collection.name = {"nb": "En begrepssamling"}
    >>> collection.publisher = "https://example.com/publishers/1"
    >>>
    >>> # Create a concept:
    >>> c = Concept()
    >>> c.identifier = "http://example.com/concepts/1"
    >>> c.term = {"name": {"nb": "inntekt", "en": "income"}}
    >>> definition = Definition()
    >>> definition.text = {"nb": "ting man skulle hatt mer av",
    >>>                    "en": "something you want more of"}
    >>> c.definition = definition
    >>>
    >>> # Add concept to collection:
    >>> collection.members.append(c)
    >>>
    >>> # get rdf representation in turtle (default)
    >>> bool(collection.to_rdf())
    True

"""
try:
    from importlib.metadata import version, PackageNotFoundError  # type: ignore
except ImportError:  # pragma: no cover
    from importlib_metadata import version, PackageNotFoundError  # type: ignore
try:
    __version__ = version(__name__)
except PackageNotFoundError:  # pragma: no cover
    __version__ = "unknown"

from .alternativformulering import AlternativFormulering
from .associativerelation import AssociativeRelation
from .betydningsbeskrivelse import Betydningsbeskrivelse
from .collection import Collection
from .concept import Concept
from .conceptrelation import ConceptRelation
from .contact import Contact
from .definition import Definition
from .genericrelation import GenericRelation
from .partitiverelation import PartitiveRelation



"""Concept module for mapping a concept to rdf.

This module contains methods for mapping a concept object to rdf
according to the
`skos-ap-no standard <https://doc.difi.no/data/begrep-skos-ap-no/#_begrep>`__

Example:
    >>> from concepttordf import Collection, Concept, Definition
    >>>
    >>> # Create collection object
    >>> collection = Collection()
    >>> collection.identifier = "http://example.com/collections/1"
    >>> collection.name = {"en": "A concept collection"}
    >>> collection.name = {"nb": "En begrepssamling"}
    >>> collection.publisher = "https://example.com/publishers/1"
    >>>
    >>> # Create a concept:
    >>> c = Concept()
    >>> c.identifier = "http://example.com/concepts/1"
    >>> c.term = {"name": {"nb": "inntekt", "en": "income"}}
    >>> definition = Definition()
    >>> definition.text = {"nb": "ting man skulle hatt mer av",
    >>>                    "en": "something you want more of"}
    >>> c.definition = definition
    >>>
    >>> # Add concept to collection:
    >>> collection.members.append(c)
    >>>
    >>> # get rdf representation in turtle (default)
    >>> bool(collection.to_rdf())
    True
"""
from typing import List

from rdflib import BNode, Graph, Literal, Namespace, RDF, RDFS, URIRef

from .concept import Concept
from .contact import Contact

DCT = Namespace("http://purl.org/dc/terms/")
SKOSXL = Namespace("http://www.w3.org/2008/05/skos-xl#")
VCARD = Namespace("http://www.w3.org/2006/vcard/ns#")
SKOS = Namespace("http://www.w3.org/2004/02/skos/core#")
SKOSNO = Namespace("http://difi.no/skosno")
DCAT = Namespace("http://www.w3.org/ns/dcat#")


class Collection:
    """A class representing a concept collection."""

    # Types
    _identifier: str
    _name: dict
    _publisher: str
    _description: dict
    _contactpoint: Contact
    _members: List[Concept]
    _dct_identifier: str

    def __init__(self) -> None:
        """Inits an object with default values."""
        self._g = Graph()
        self._members = []

    @property
    def identifier(self) -> str:
        """Identifier attribute.

        Returns:
            a string holding a valid uri
        """
        return self._identifier

    @identifier.setter
    def identifier(self, uri: str) -> None:
        self._identifier = uri

    @property
    def name(self) -> dict:
        """Name attribute."""
        return self._name

    @name.setter
    def name(self, name: dict) -> None:
        self._name = name

    @property
    def publisher(self) -> str:
        """Publisher attribute."""
        return self._publisher

    @publisher.setter
    def publisher(self, publisher: str) -> None:
        self._publisher = publisher

    @property
    def description(self) -> dict:
        """Description attribute."""
        return self._description

    @description.setter
    def description(self, description: dict) -> None:
        self._description = description

    @property
    def contactpoint(self) -> Contact:
        """Contactpoint attribute."""
        return self._contactpoint

    @contactpoint.setter
    def contactpoint(self, contact: Contact) -> None:
        self._contactpoint = contact

    @property
    def members(self) -> List[Concept]:
        """Members attribute."""
        return self._members

    @members.setter
    def members(self, members: List[Concept]) -> None:
        self._members = members

    @property
    def dct_identifier(self) -> str:
        """Get for dct_identifier."""
        return self._dct_identifier

    @dct_identifier.setter
    def dct_identifier(self, dct_identifier: str) -> None:
        """Set for dct_identifier."""
        self._dct_identifier = dct_identifier

    # ----------------------------------------------

    def to_rdf(self, format: str = "text/turtle", includeconcepts: bool = True) -> str:
        """Maps the collection to rdf.

        Args:
            format: the format of the serialization
            includeconcepts: if true details of concepts are included

        Returns:
            serialization as a string according to format
        """
        return self._to_graph(includeconcepts=includeconcepts).serialize(format=format)

    # ---

    def _to_graph(self, includeconcepts: bool = True) -> Graph:
        """Transforms the collection to an rdf graph.

        Args:
            includeconcepts: if true details of concepts are included

        Returns:
            an RDF graph representing the collection.
        """
        self._add_collection_to_graph()

        if includeconcepts:
            for concept in self.members:
                self._g += concept._to_graph()

        return self._g

    # ---

    def _add_collection_to_graph(self) -> None:
        """Adds the collection to the Graph _g."""
        self._g.bind("dct", DCT)
        self._g.bind("skos", SKOS)
        self._g.bind("skosxl", SKOSXL)
        self._g.bind("vcard", VCARD)
        self._g.bind("skosno", SKOSNO)
        self._g.bind("dcat", DCAT)

        self._g.add((URIRef(self.identifier), RDF.type, SKOS.Collection))

        # dct:identifier
        if getattr(self, "dct_identifier", None):
            self._g.add(
                (
                    URIRef(self.identifier),
                    DCT.identifier,
                    Literal(self.dct_identifier),
                )
            )

        # publisher
        self._g.add((URIRef(self.identifier), DCT.publisher, URIRef(self.publisher)))

        # name
        if getattr(self, "name", None):
            for _key in self.name:
                self._g.add(
                    (
                        URIRef(self.identifier),
                        RDFS.label,
                        Literal(self.name[_key], lang=_key),
                    )
                )

        # description
        if getattr(self, "description", None):
            for _key in self.description:
                self._g.add(
                    (
                        URIRef(self.identifier),
                        DCT.description,
                        Literal(self.description[_key], lang=_key),
                    )
                )

        # contactPoint
        if getattr(self, "contactpoint", None):
            contact = self.contactpoint
            contactPoint = BNode()
            for _s, p, o in contact._to_graph().triples((None, None, None)):
                self._g.add((contactPoint, p, o))
            self._g.add((URIRef(self.identifier), DCAT.contactPoint, contactPoint))

        # members
        if getattr(self, "members", None):
            for concept in self.members:
                self._g.add(
                    (URIRef(self.identifier), SKOS.member, URIRef(concept.identifier))
                )



"""Module for helper class Definition."""
from rdflib import Namespace

from .betydningsbeskrivelse import Betydningsbeskrivelse

SKOSNO = Namespace("https://data.norge.no/vocabulary/skosno#")


class Definition(Betydningsbeskrivelse):
    """A class representing a definition."""

    def __init__(self) -> None:
        """Inits an object with default values."""
        super().__init__()



"""Concept module for mapping class associative relation to rdf."""
from __future__ import annotations

from typing import List, TYPE_CHECKING

from rdflib import Graph, Literal, Namespace, RDF, URIRef

from .conceptrelation import ConceptRelation

if TYPE_CHECKING:  # pragma: no cover
    from .concept import Concept

SKOS = Namespace("http://www.w3.org/2004/02/skos/core#")
SKOSNO = Namespace("https://data.norge.no/vocabulary/skosno#")
DCT = Namespace("http://purl.org/dc/terms/")
XSD = Namespace("http://www.w3.org/2001/XMLSchema#")


class AssociativeRelation(ConceptRelation):
    """A class representing a associative concept relation.

    Attributes:
        description: dictionary where key is language and value describing relation
        associatedconcepts: list of related concepts
    """

    _description: dict
    _associatedconcepts: List[Concept]

    def __init__(self) -> None:
        """Inits an object with default values."""
        self._associatedconcepts = list()
        super().__init__()

    @property
    def description(self) -> dict:
        """Description attribute."""
        return self._description

    @description.setter
    def description(self, description: dict) -> None:
        """Description attribute.

        Args:
            description: a dictionary with key language code and value text.
        """
        self._description = description

    @property
    def associatedconcepts(self) -> List[Concept]:
        """Associatedconcepts attributes.

        Returns:
            a list of related concepts
        """
        return self._associatedconcepts

    # ---

    def _to_graph(self) -> Graph:

        self._add_relation_to_graph()

        return self._g

    # ---
    def _add_relation_to_graph(self) -> None:

        super(AssociativeRelation, self)._add_relation_to_graph()

        self._g.add((self._relation, RDF.type, SKOSNO.AssosiativRelasjon))

        # description
        if getattr(self, "description", None):
            for key in self.description:
                self._g.add(
                    (
                        self._relation,
                        DCT.description,
                        Literal(self.description[key], lang=key),
                    )
                )

        # associatedconcepts
        if getattr(self, "associatedconcepts", None):
            for ac in self.associatedconcepts:
                self._g.add((self._relation, SKOS.related, URIRef(ac.identifier)))



"""Concept module for mapping abstract class Betydningsbeskrivelse to rdf."""
from abc import ABC
from datetime import date
from enum import Enum


class RelationToSource(Enum):
    """An enum representing relations to source."""

    sitatFraKilde = "quoteFromSource"
    basertPaKilde = "basedOnSource"
    egendefinert = "noSource"


class Betydningsbeskrivelse(ABC):
    """A class representing a concept.

    Attributes:
        text:
        remark:
        scope:
        relationtosource:
        source:
        modified:
        example:
    """

    def __init__(self) -> None:
        """Inits an object with default values."""
        pass

    @property
    def text(self) -> dict:
        """Text attribute.

        Returns:
            a dict with key as language code (str) and value text (str)
        """
        return self._text

    @text.setter
    def text(self, text: dict) -> None:
        self._text = text

    @property
    def remark(self) -> dict:
        """Remark attribute.

        Returns:
            a dict with key as language code (str) and value remark (str)
        """
        return self._remark

    @remark.setter
    def remark(self, remark: dict) -> None:
        self._remark = remark

    @property
    def scope(self) -> dict:
        """Scope attribute.

        Returns:
            a dict with key as language code (str) and value scope (str)
        """
        return self._scope

    @scope.setter
    def scope(self, scope: dict) -> None:
        self._scope = scope

    @property
    def relationtosource(self) -> str:
        """Relationtosource attribute.

        Returns:
            a relationtype to source (str)
        """
        return self._relationtosource

    @relationtosource.setter
    def relationtosource(self, relationtosource: str) -> None:
        self._relationtosource = relationtosource

    @property
    def source(self) -> dict:
        """Source attribute.

        Returns:
            a dict with key as language code (str) and value remark (str)
        """
        return self._source

    @source.setter
    def source(self, source: dict) -> None:
        self._source = source

    @property
    def modified(self) -> date:
        """Source attribute.

        Returns:
            the date on which the item was last updated
        """
        return self._modified

    @modified.setter
    def modified(self, modified: date) -> None:
        self._modified = modified

    @property
    def example(self) -> dict:
        """Example attribute.

        Returns:
            a dict with key as language code (str) and value example (str)
        """
        return self._example

    @example.setter
    def example(self, example: dict) -> None:
        self._example = example



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['concepttordf']

package_data = \
{'': ['*']}

install_requires = \
['rdflib>=6.0.0,<7.0.0']

setup_kwargs = {
    'name': 'concepttordf',
    'version': '2.0.2',
    'description': 'A library for mapping a concept collection to rdf',
    'long_description': '\n# concepttordf\n\n[!Tests](https://github.com/Informasjonsforvaltning/concepttordf/workflows/Tests/badge.svg)\n[![codecov](https://codecov.io/gh/Informasjonsforvaltning/concepttordf/branch/master/graph/badge.svg)](https://codecov.io/gh/Informasjonsforvaltning/concepttordf)\n[![PyPI](https://img.shields.io/pypi/v/concepttordf.svg)](https://pypi.org/project/concepttordf/)\n[![Read the Docs](https://readthedocs.org/projects/concepttordf/badge/)](https://concepttordf.readthedocs.io/)\n\nA small Python library for mapping a concept collection to the [skos-ap-no specification](https://doc.difi.no/data/begrep-skos-ap-no/).\n\n## Usage\n\n### Install\n\n```Shell\n% pip install concepttordf\n```\n\n### Getting started\n\nTo create a SKOS-AP-NO concept collection:\n\n```Shell\nfrom concepttordf import Collection, Concept, Definition\n\n# Create collection object\ncollection = Collection()\ncollection.identifier = "http://example.com/collections/1"\ncollection.name = {"en": "A concept collection"}\ncollection.name = {"nb": "En begrepssamling"}\ncollection.publisher = "https://example.com/publishers/1"\n\n# Create a concept:\nc = Concept()\nc.identifier = "http://example.com/concepts/1"\nc.term = {"name": {"nb": "inntekt", "en": "income"}}\ndefinition = Definition()\ndefinition.text = {"nb": "ting man skulle hatt mer av",\n                   "en": "something you want more of"}\nc.definition = definition\n\n# Add concept to collection:\ncollection.members.append(c)\n\n# get rdf representation in turtle (default)\nrdf = collection.to_rdf()\nprint(rdf)\n```\n\nWill print the concept according to the specification:\n\n```Shell\n@prefix dcat: <http://www.w3.org/ns/dcat#> .\n@prefix dct: <http://purl.org/dc/terms/> .\n@prefix ns1: <https://data.norge.no/vocabulary/skosno#> .\n@prefix rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#> .\n@prefix rdfs: <http://www.w3.org/2000/01/rdf-schema#> .\n@prefix skos: <http://www.w3.org/2004/02/skos/core#> .\n@prefix skosno: <http://difi.no/skosno> .\n@prefix skosxl: <http://www.w3.org/2008/05/skos-xl#> .\n@prefix vcard: <http://www.w3.org/2006/vcard/ns#> .\n@prefix xml: <http://www.w3.org/XML/1998/namespace> .\n@prefix xsd: <http://www.w3.org/2001/XMLSchema#> .\n\n<http://example.com/collections/1> a skos:Collection ;\n    rdfs:label "En begrepssamling"@nb ;\n    dct:publisher <https://example.com/publishers/1> ;\n    skos:member <http://example.com/concepts/1> .\n\n<http://example.com/concepts/1> a skos:Concept ;\n    ns1:betydningsbeskrivelse [ a ns1:Definisjon ;\n            rdfs:label "something you want more of"@en,\n                "ting man skulle hatt mer av"@nb ] ;\n    skosxl:prefLabel [ a skosxl:Label ;\n            skosxl:literalForm "income"@en,\n                "inntekt"@nb ] .\n\n```\n\n## Development\n\n### Requirements\n\n- [pipx](https://pipxproject.github.io/pipx/) (recommended)\n- [pyenv](https://github.com/pyenv/pyenv) (recommended)\n- [poetry](https://python-poetry.org/)\n- [nox](https://nox.thea.codes/en/stable/)\n\n```Shell\n% pipx install poetry\n% pipx install nox\n% pipx inject nox nox-poetry\n```\n\n### Install dev tools\n\n```Shell\n% git clone https://github.com/Informasjonsforvaltning/concepttordf.git\n% cd concepttordf\n% pyenv install 3.8.2\n% pyenv install 3.9.6\n% pyenv local 3.8.2 3.9.6\n% poetry install\n```\n\n### Run all tests\n\n```Shell\n% nox\n```\n\n### Run all tests with coverage reporting\n\n```Shell\n% nox -rs tests\n```\n\n### Debugging\n\nYou can enter into [Pdb](https://docs.python.org/3/library/pdb.html) by passing `--pdb` to pytest:\n\n```Shell\nnox -rs tests -- --pdb\n```\n\nYou can set breakpoints directly in code by using the function `breakpoint()`.\n',
    'author': 'Stig B. Dørmænen',
    'author_email': 'stigbd@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/Informasjonsforvaltning/concepttordf',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<3.11',
}


setup(**setup_kwargs)



"""Concept module for mapping class partitive relation to rdf."""
from __future__ import annotations

from typing import List, TYPE_CHECKING

from rdflib import Graph, Literal, Namespace, RDF, URIRef

from .conceptrelation import ConceptRelation

if TYPE_CHECKING:  # pragma: no cover
    from .concept import Concept

SKOSNO = Namespace("https://data.norge.no/vocabulary/skosno#")
DCT = Namespace("http://purl.org/dc/terms/")
XSD = Namespace("http://www.w3.org/2001/XMLSchema#")


class PartitiveRelation(ConceptRelation):
    """A class representing a partitive concept relation.

    Attributes:
        criterium: dictionary where key is language an value describing relation
        partconcepts: list of related concepts
    """

    _criterium: dict
    _partconcepts: List[Concept]
    _is_part_of_concepts: List[Concept]

    def __init__(self) -> None:
        """Inits an object with default values."""
        self._partconcepts = list()
        self._is_part_of_concepts = list()
        super().__init__()

    @property
    def criterium(self) -> dict:
        """Criterium attribute."""
        return self._criterium

    @criterium.setter
    def criterium(self, criterium: dict) -> None:
        """Criterium attribute.

        Args:
            criterium: a dictionary with key language code and value text.
        """
        self._criterium = criterium

    @property
    def partconcepts(self) -> List[Concept]:
        """Partconcepts attributes.

        Returns:
            a list of related concepts
        """
        return self._partconcepts

    @property
    def is_part_of_concepts(self) -> List[Concept]:
        """Is_part_of_concepts attributes.

        Returns:
            a list of concepts that this concept is part of
        """
        return self._is_part_of_concepts

    # ---

    def _to_graph(self) -> Graph:

        self._add_relation_to_graph()

        return self._g

    # ---
    def _add_relation_to_graph(self) -> None:

        super(PartitiveRelation, self)._add_relation_to_graph()

        self._g.add((self._relation, RDF.type, SKOSNO.PartitivRelasjon))

        # criterium
        if getattr(self, "criterium", None):
            for key in self.criterium:
                self._g.add(
                    (
                        self._relation,
                        DCT.description,
                        Literal(self.criterium[key], lang=key),
                    )
                )

        # partconcepts
        if getattr(self, "partconcepts", None):
            for ac in self.partconcepts:
                self._g.add((self._relation, DCT.hasPart, URIRef(ac.identifier)))

        # is_part_of_concepts
        if getattr(self, "is_part_of_concepts", None):
            for ac in self.is_part_of_concepts:
                self._g.add((self._relation, DCT.isPartOf, URIRef(ac.identifier)))
